*****************************************************************************************
*                                                                                       *
*  DragonsLair3.Shared.s :  Library for shared functions.                               *
*                                                                                       *
*****************************************************************************************
*  Authors : Antoine VIGNAU / Olivier ZARDINI  *  Brutal Deluxe Software  *   Dec 2021  *
*****************************************************************************************

************************************************************
*  ToolInitialize() :  Initialize Tools layer (OUT:A=@DP). *
************************************************************
ToolInitialize      _TLStartUp                      ; Start Tools
                    PHA
                    _MMStartUp                      ; Start Memory Manager Tool Set
                    PLA
                    STA        myID                 ; Get current ID
*--
                    _MTStartUp                      ; Start Miscellaneous Tool Set
*--
                    PushLong   #0                   ; Allocate Page Direct in Bank 00
                    PushLong   #$000100
                    PushWord   myID
                    PushWord   #$C005               ; Fixed, Page Aligned, Locked, Unpurgeable
                    PushLong   #0
                    _NewHandle
                    PLX                             ; Handle Low  Address
                    PLA                             ; Handle High Address (00XX bank)
                    XBA
                    STA        TI_1+2
TI_1                LDAL       $000000,X            ; Get Low Address  A=??/XXXX
                    PHA
*--
                    _SoundStartUp                   ; Start Sound Tool Set
                    BCS        TI_NoSound
                    PushLong   #0
                    PushWord   #11
                    _GetVector
                    PullLong   BackupSoundVector
                    PushWord   #11
                    PushLong   #EntrySoundVector
                    _SetVector
                    BRA        TI_CompactMemory
TI_NoSound          INC        SoundToolError       ; No Sound Available                    
*--
TI_CompactMemory    PushLong   #0                   ; Compact Memory
                    PushLong   #$8FFFFF
                    PushWord   myID
                    PushWord   #%11000000_00000000
                    PushLong   #0
                    _NewHandle
                    _DisposeHandle
                    _CompactMem
*--
                    PushLong   #0                   ; Ask Shadowing Screen ($8000 bytes from $01/2000)
                    PushLong   #$8000
                    PushWord   myID
                    PushWord   #%11000000_00000011
                    PushLong   #$012000
                    _NewHandle
                    PLA
                    PLA
*--
                    RTS


************************************************
*  ToolTerminate() :  Free Memory, Stop Tools. *
************************************************
ToolTerminate       LDA        SoundToolError       ; Stop Tools
                    BNE        TT_MiscTool
                    PushWord   #11
                    PushLong   BackupSoundVector    ; Put back Sound Vector
                    _SetVector
                    _SoundShutDown
*--
TT_MiscTool        _MTShutDown
*--
                    PushWord   myID
                    _DisposeAll
*--               
                    PushWord   myID
                    _MMShutDown
*--               
                    _TLShutDown
*--
                    RTS
               
*----------   Tools Variables   ---------
myID                ds         2                    ; ID of this Program in memory
SoundToolError      HEX        0000                 ; 1 if sound Error
BackupSoundVector   DS         4                    ; Interrupt Sound Vector backup


****************************************************
*  EntrySoundVector() :  Sound Interrupt Function. *
****************************************************
EntrySoundVector    CLC
                    RTL


********************************************************
*  BackupEnvironment() :  Backup Environment Settings. *
********************************************************
BackupEnvironment   SEP        #$20                 ; Backup Environment values (color, border...)
*--
                    LDAL       TBCOLOR              ; Text + Background Color
                    STA        BE_C022
*--            
                    LDAL       NEWVIDEO             ; Apple IIgs Graphic Mode
                    STA        BE_C029
*--            
                    LDAL       CLOCKCTL             ; Border Color
                    STA        BE_C034
*--            
                    LDAL       SHADOW               ; Shadowing
                    STA        BE_C035
*--
                    LDAL       RD80VID              ; 80 Column status
                    STA        BE_C01F
*--
                    REP        #$30
                    RTS


**********************************************************
*  RestoreEnvironment() :  Restore Environment Settings. *
**********************************************************
RestoreEnvironment  SEP        #$20                 ; Restore Environment values (color, border...)
*--
                    LDA        BE_C035              ; Shadowing
                    STAL       SHADOW
*--            
                    LDA        BE_C034              ; Border Color
                    STAL       CLOCKCTL
*--            
                    LDA        BE_C029              ; Apple IIgs Graphic Mode
                    STAL       NEWVIDEO
*--            
                    LDA        BE_C022              ; Text + Background Color
                    STAL       TBCOLOR
*--
                    LDA        BE_C01F              ; Restaure 80 Column if needed
                    ASL                             ; Bit #7=1 if 80 column
                    BCC        RE_End
                    STAL       SET80VID             ; Set 80 column mode      
*--
RE_End              REP        #$30
                    RTS

*--------------   Environment Variables   -------------
BE_C01F             HEX        00                   ; 80 Column status
BE_C022             HEX        00                   ; Background + Text Color
BE_C029             HEX        00                   ; Linearization of the Graphic Page
BE_C034             HEX        00                   ; Border Color
BE_C035             HEX        00                   ; Shadowing


***********************************************************************************
*  AllocOneBank() :  Allocate 1 Bank of 64 KB (OUT:A=XX/00,OUT:Carry=SetOnError). *
***********************************************************************************
AllocOneBank        PushLong   #0
                    PushLong   #$10000              ; 64 KB
                    PushWord   myID                 ; ID of my current Program
                    PushWord   #%11000000_00011100
                    PushLong   #0
                    _NewHandle
                    PLX                             ; Handle Low Address
                    PLA                             ; Handle High Address (00XX)
                    XBA
                    STA        AOB_1+2
AOB_1               LDAL       $000001,X            ; Get Bank Address (A=XX/00)
                    RTS


***************************************************************************************************************
*  MemoryAllocation() :  Allocate all required Banks of 64 KB (IN:A=MaxBank,IN:X=@TableBank,OUT:A=AllocBank). *
***************************************************************************************************************
MemoryAllocation    ASL                             ; Maximum of # Bank required x 2
                    STA        MA_3+1
                    STX        MA_2+1
                    LDX        #$0000
*--                    
MA_1                PHX                             ; Allocate one Bank of 64 KB
                    JSR        AllocOneBank
                    PLX
                    BCS        MA_End               ; Error, no more memory available
*--
MA_2                STA        $AAAA,X              ; Store Bank Number in the Table
                    INX
                    INX
*--                    
MA_3                CPX        #$AAAA               ; Don't allocate more than required
                    BMI        MA_1                
*---                    
MA_End              TXA                             ; Number of allocated Bank x 2
                    LSR      
                    RTS


********************************************************
*  WaitForKey() :  Wait for a Key pressed (OUT:A=Key). *
********************************************************
WaitForKey          SEP        #$20                 ; Wait for a Key Press
WFK_1               LDAL       KBDREAD
                    BPL        WFK_1
                    STAL       KBDSTRB
                    REP        #$30
                    AND        #$007F               ; Clear Bit #7
                    RTS


***********************************************
*  ClearKDBStrobe() :  Clear Keyboard Strobe. *
***********************************************
ClearKDBStrobe      SEP        #$20                 ; Reset any Key Pressed
                    STAL       KBDSTRB
                    REP        #$30
                    RTS


*************************************************************************
*  HasKeyPressed() :  Check for a Key pressed (OUT:Carry=SetIfPressed). *
*************************************************************************
HasKeyPressed       SEP        #$20                 ; Wait for a Key Press
                    CLC                             ; Init as 'Not Pressed'
                    LDAL       KBDREAD
                    BPL        HKP_End
                    STAL       KBDSTRB
                    SEC
HKP_End             REP        #$30
                    RTS


*****************************************************
*  ResetTimerSec() :  Start the Timer (in Seconds). *
*****************************************************
ResetTimerSec       PHA                             ; Space for result
                    PHA
                    PHA
                    PHA
                    _ReadTimeHex                    ; Read current Time/Date in HEX Format
                    PLA
                    STA        RTS_SECOND_MINUTE
                    PLA
                    STA        RTS_HOUR_YEAR
                    PLA
                    STA        RTS_DAY_MONTH
                    PLA
                    STA        RTS_NULL_WEEKDAY                                                
                    RTS

*----------   ResetTimerSec Variables   ---------
RTS_SECOND_MINUTE   HEX        00,00                ; 0-59  0-59
RTS_HOUR_YEAR       HEX        00,00                ; 0-23  Year-1900  
RTS_DAY_MONTH       HEX        00,00                ; 0-11  0-30
RTS_NULL_WEEKDAY    HEX        00,00                ; 1(Sunday)-7  NULL


*********************************************************************************
*  GetTimerSec() :  Compute the number of seconds since the Reset (OUT:A=#Sec). *
*********************************************************************************
GetTimerSec         PHA                             ; Space for result
                    PHA
                    PHA
                    PHA
                    _ReadTimeHex                    ; Read current Time/Date in HEX Format        
                    PLA
                    STA        GTS_SECOND_MINUTE
                    PLA
                    PLA
                    PLA
*--
                    LDA        GTS_SECOND_MINUTE    ; Compute difference
                    CMP        RTS_SECOND_MINUTE
                    BEQ        GetTimerSec_Same
*--
                    SEP        #$20
                    LDA        GTS_SECOND_MINUTE
                    CMP        RTS_SECOND_MINUTE
                    BMI        GetTimerSec_Min
                    SEC
                    SBC        RTS_SECOND_MINUTE
                    BRA        GetTimerSec_End  
*--                    
GetTimerSec_Min     LDA        #$3C                 ; 60 sec
                    SEC
                    SBC        RTS_SECOND_MINUTE
                    CLC
                    ADC        GTS_SECOND_MINUTE
*--                    
GetTimerSec_End     REP        #$30
                    AND        #$00FF               ; 0-59 sec 
                    RTS
*--
GetTimerSec_Same    LDA        #$0000               ; In the Same second
                    RTS
                    
*----------   GetTimerSec Variables   ---------
GTS_SECOND_MINUTE   HEX        00,00                ; 0-59  0-59


**********************************************************************************************
*  LoadFile() :  Load one File into memory (IN:X=@FilePath,IN:A=BankXX/00,OUT:Carry=Status). *
**********************************************************************************************
LoadFile            STX        LF_gsosOPEN+4        ; X = File Path 
                    STA        LF_gsosREAD+5        ; A = Bank XX/00
*--
LF_Open             JSL        GSOS                 ; Open File
                    dw         $2010                ; Code for command 'Open'
                    adrl       LF_gsosOPEN          ; Parameters Table Address (Open in Read Mode)
                    BCS        LF_ErrorEnd
                    LDA        LF_gsosOPEN+2
                    STA        LF_gsosGETEOF+2
                    STA        LF_gsosREAD+2
                    STA        LF_gsosCLOSE+2
*--
LF_GetSize          JSL        GSOS                 ; Get File Size
                    dw         $2019                ; Code for command 'GetFileSize'
                    adrl       LF_gsosGETEOF        ; Parameters Table Address
                    LDA        LF_gsosGETEOF+4
                    STA        LF_gsosREAD+8
                    LDA        LF_gsosGETEOF+6
                    STA        LF_gsosREAD+10
*--
LF_Read             JSL        GSOS                 ; Read File Content
                    dw         $2012                ; Code for command 'Read'
                    adrl       LF_gsosREAD          ; Parameters Table Address
                    BCS        LF_Error
*--
LF_Close            JSL        GSOS                 ; Close File
                    dw         $2014                ; Code for command 'Close'
                    adrl       LF_gsosCLOSE         ; Parameters Table Address
*--
LF_End              CLC                             ; No Error
                    LDA        LF_gsosGETEOF+4      ; A = File Size
                    RTS
*----
LF_Error            JSL        GSOS                 ; Close File
                    dw         $2014                ; Code for command 'Close'
                    adrl       LF_gsosCLOSE         ; Parameters Table Address
*--
LF_ErrorEnd         SEC                             ; Error
                    RTS
                         
*----------   LoadFile Variables   ---------
LF_gsosOPEN         dw         2                    ; Number of parameters (Open in Read Mode)
                    ds         2                    ; +2 File Descriptor
                    adrl       LDF_FileData_HDrive  ; +4 File Pathname (init with an existing file path, so we only have to change 2 bytes)

LF_gsosGETEOF       dw         2                    ; Number of parameters
                    ds         2                    ; +2 File Descriptor
                    ds         4                    ; +2 eof

LF_gsosREAD         dw         4                    ; Number of parameters
                    ds         2                    ; +2  File Descriptor
                    ds         4                    ; +4  Data Buffer Address
                    ds         4                    ; +8  requestCount
                    ds         4                    ; +12 transferCount
                    
LF_gsosCLOSE        dw         1                    ; Number of parameters
                    ds         2                    ; +2 File Descriptor


*******************************************************************************************
*  IsFileExist() :  Check if a file already exist on disk (IN:A=FilePath,OUT:A=1IfExist). *
*******************************************************************************************
IsFileExist         STA        IFE_gsosOPEN+4       ; A = File Path 
*--
IFE_Open            JSL        GSOS                 ; Open File
                    dw         $2010                ; Code for command 'Open'
                    adrl       IFE_gsosOPEN         ; Parameters Table Address (Open in Read Mode)
                    BCS        IFE_ErrorEnd
                    LDA        IFE_gsosOPEN+2
                    STA        IFE_gsosCLOSE+2
*--
IFE_Close           JSL        GSOS                 ; Close File
                    dw         $2014                ; Code for command 'Close'
                    adrl       IFE_gsosCLOSE        ; Parameters Table Address
*--
IFE_End             LDA        #$0001               ; File exist
                    RTS
*--
IFE_ErrorEnd        LDA        #$0000               ; File doesn't exist
                    RTS

*----------   IsFileExist Variables   ---------
IFE_gsosOPEN        dw         2                    ; Number of parameters (Open in Read Mode)
                    ds         2                    ; +2 File Descriptor
                    adrl       LDF_FileData_HDrive  ; +4 File Pathname (init with an existing file path, so we only have to change 2 bytes)
                    
IFE_gsosCLOSE       dw         1                    ; Number of parameters
                    ds         2                    ; +2 File Descriptor


***********************************************************************
*  ExitToLauncher() :  Quit program and go back to Finder / Launcher. *
***********************************************************************
ExitToLauncher      JSL        GSOS                 ; Quit Program
                    dw         $2029
                    adrl       gsosQUIT               
               
*----------   GS/OS Calls Variables   ---------
gsosQUIT            dw         2                    ; Number of parameters
                    ds         4                    ; +2 pathname
                    ds         2                    ; +6 flags


********************************************************************************************
*  CreateLogFile() :  Create the Log file on disk (IN:A=LogFilePath,OUT:Carry=SetOnError). *
********************************************************************************************
CreateLogFile       STA        CLF_gsosCREATE+2     ; A = File Path 
                    STA        CLF_gsosDESTROY+2
                    STA        ALL_LogFilePath
*----
                    JSL        GSOS                 ; Delete File
                    dw         $2002                ; Code for command 'Destroy'
                    adrl       CLF_gsosDESTROY      ; Parameters Table Address   
*----                    
                    SEP        #$20
                    LDA        #$E3
                    STA        CLF_gsosCREATE+6     ; File Access
                    LDA        #$0004
                    STA        CLF_gsosCREATE+8     ; File Type (Text)
                    LDA        #$01
                    STA        CLF_gsosCREATE+14    ; Storage Type (Standard)
                    REP        #$30
                    STZ        CLF_gsosCREATE+10    ; File AuxType
                    STZ        CLF_gsosCREATE+12
*--
                    JSL        GSOS                 ; Create File
                    dw         $2001                ; Code for command 'Create'
                    adrl       CLF_gsosCREATE       ; Parameters Table Address                    
*----
                    RTS               

*----------   CreateLogFile Variables   ---------
LogFilePath         strl       '1/LogFile.txt'
LOG_LINE_1          ASC        'Log Line #1',00
LOG_LINE_2          ASC        'Log Line #2',00
LOG_LINE_3          ASC        'Log Line #3',00
LOG_LINE_4          ASC        'Log Line %1Y-%2Y-%4Y 123',00

STRING_1            ASC        'SubString1',00
STRING_2            ASC        'SubString2',00
VALUEHEX            HEX        0020E1FF

CLF_gsosDESTROY     dw         1                    ; Number of parameters
                    adrl       LogFilePath          ; +2  File Pathname (init with an existing file path, so we only have to change 2 bytes)
                    
CLF_gsosCREATE      dw         5                    ; Number of parameters
                    adrl       LogFilePath          ; +2  File Pathname (init with an existing file path, so we only have to change 2 bytes)
                    ds         2                    ; +6  File Access
                    ds         2                    ; +8  File Type
                    ds         4                    ; +10 File AuxType       
                    ds         2                    ; +14 Storage Type


**************************************************************************************************************
*  AddLogLine() :  Add one line to the Log file (IN:A=LogLine,IN:X=ParamX,IN:Y=ParamY,OUT:Carry=SetOnError). *
**************************************************************************************************************
AddLogLine          STA        ALL_Prepare_1+1      ; A = @ Line
                    STA        ALL_PrepareParam+1
                    STA        ALL_PrepareNum_1+1
                    STA        ALL_PrepareNum_2+1
                    STA        ALL_PrepareNum_4+1
                    STA        ALL_PrepareString+1
*-
                    STX        ALL_PrepareX_1+1     ; X Parameter for Log Line
                    STX        ALL_PrepareNum1X+1
                    STX        ALL_PrepareNum2X_1+1
                    STX        ALL_PrepareNum4X_1+1
                    STX        ALL_PrepareStrX_1+1
                    INX
                    INX
                    STX        ALL_PrepareNum4X_2+1
*-
                    STY        ALL_PrepareY_1+1     ; Y Parameter for Log Line
                    STY        ALL_PrepareNum1Y+1
                    STY        ALL_PrepareNum2Y_1+1
                    STY        ALL_PrepareNum4Y_1+1
                    STY        ALL_PrepareStrY_1+1
                    INY
                    INY
                    STY        ALL_PrepareNum4Y_2+1
                                                  
*----  Get Current Time / Date  ----
ALL_GetTimeDate     PushLong   #ALL_TimeBuffer      ; Get current Time/Date
                    _ReadAsciiTime
                    LDX        #$0012               ; Move High Ascii to Ascii
ALL_GetTimeDate_1   LDA        ALL_TimeBuffer,X
                    AND        #$7F7F
                    STA        ALL_TimeBuffer,X
                    DEX
                    DEX
                    BPL        ALL_GetTimeDate_1
*-
ALL_CleanTimeDate   SEP        #$30                 ; Replace Space with 0
                    LDX        #$30                 ; '0'
                    LDA        ALL_TimeBuffer
                    CMP        #$20                 ; ' '
                    BNE        ALL_CleanTimeDate_1
                    STX        ALL_TimeBuffer
ALL_CleanTimeDate_1 LDA        ALL_TimeBuffer+3
                    CMP        #$20                 ; ' '
                    BNE        ALL_CleanTimeDate_2
                    STX        ALL_TimeBuffer+3           
ALL_CleanTimeDate_2 REP        #$30

*----  Prepare Log Line (Replace X and Y parameters, Compute Length)  ----
                    LDX        #$0000               ; Replace Log Line parameters
                    LDY        #$0000               ; X=Src, Y=Dst
*--
ALL_Prepare_0       SEP        #$20 
ALL_Prepare_1       LDA        $AAAA,X
                    INX
                    CMP        #$00
                    BEQ        ALL_GotoEnd          ; \0 is the End of Line
                    CMP        #$25                 ; '%'
                    BEQ        ALL_PrepareParam        
                    STA        ALL_LogBuffer,Y
                    INY
                    BRA        ALL_Prepare_1
ALL_GotoEnd         JMP        ALL_PrepareEnd
*----                             
ALL_PrepareParam    LDA        $AAAA,X              ; %1X|Y %2X|Y %4X|Y %SX %SX %X %Y
                    INX
                    CMP        #$31                 ; '1'
                    BEQ        ALL_PrepareNum_1
                    CMP        #$32                 ; '2'
                    BEQ        ALL_PrepareNum_2
                    CMP        #$34                 ; '4'
                    BEQ        ALL_GotoPrepNum_4
                    CMP        #$73                 ; 's'
                    BEQ        ALL_GotoPrepString
                    CMP        #$53                 ; 'S'
                    BEQ        ALL_GotoPrepString
                    CMP        #$58                 ; 'X'
                    BEQ        ALL_PrepareX 
                    CMP        #$59                 ; 'Y'
                    BEQ        ALL_PrepareY
*--
                    STA        ALL_LogBuffer+1,Y    ; Unknown % sequence
                    LDA        #$25                 ; '%'
                    STA        ALL_LogBuffer,Y
                    INY
                    INY
                    BRA        ALL_Prepare_1
ALL_GotoPrepNum_4   JMP        ALL_PrepareNum_4
ALL_GotoPrepString  JMP        ALL_PrepareString
*----
ALL_PrepareX        REP        #$30                 ; %X : Insert X Data 16 bit
ALL_PrepareX_1      LDA        #$BBBB
                    XBA                             ; -> Big Endian
                    STA        ALL_ParamValueHex
                    JSR        ALL_InsertHexaData2
                    BRA        ALL_Prepare_0
*----
ALL_PrepareY        REP        #$30                 ; %Y : Insert Y Data 16 bit
ALL_PrepareY_1      LDA        #$CCCC
                    XBA                             ; -> Big Endian
                    STA        ALL_ParamValueHex
                    JSR        ALL_InsertHexaData2                  
                    BRA        ALL_Prepare_0
*----                 
                    mx         %10
ALL_PrepareNum_1    LDA        $AAAA,X              ; %1 : Insert 8 bit from Address X / Y
                    INX
                    CMP        #$59                 ; 'Y'
                    BEQ        ALL_PrepareNum1Y
                    CMP        #$58                 ; 'X'
                    BNE        ALL_PrepareNumUnk_1
*--     
ALL_PrepareNum1X    LDA        $BBBB                ; '%1X' : Copy 1 byte from address [X]
                    STA        ALL_ParamValueHex 
                    REP        #$30
                    JSR        ALL_InsertHexaData1  ; Add 1 byte in the String                    
                    JMP        ALL_Prepare_0
*--     
ALL_PrepareNum1Y    LDA        $CCCC                ; '%1Y' : Copy 1 byte from address [Y]
                    STA        ALL_ParamValueHex 
                    REP        #$30
                    JSR        ALL_InsertHexaData1  ; Add 1 byte in the String                    
                    JMP        ALL_Prepare_0
*--
                    mx         %10
ALL_PrepareNumUnk_1 STA        ALL_LogBuffer+2,Y    ; Unknown Sequence %1 sequence
                    LDA        #$25                 ; '%'
                    STA        ALL_LogBuffer,Y
                    LDA        #$31                 ; '1'
                    STA        ALL_LogBuffer+1,Y
                    INY
                    INY
                    INY
                    JMP        ALL_Prepare_1
*----
ALL_PrepareNum_2    LDA        $AAAA,X              ; %2 : Insert 16 bit from Address X / Y
                    INX
                    CMP        #$59                 ; 'Y'
                    BEQ        ALL_PrepareNum2Y
                    CMP        #$58                 ; 'X'
                    BNE        ALL_PrepareNumUnk_2
*--     
ALL_PrepareNum2X    REP        #$30                 ; '%2X' : Copy 2 bytes from address [X]
ALL_PrepareNum2X_1  LDA        $BBBB                
                    XBA                             ; -> Big Endian
                    STA        ALL_ParamValueHex 
                    JSR        ALL_InsertHexaData2  ; Add 2 bytes in the String                    
                    JMP        ALL_Prepare_0
*--     
ALL_PrepareNum2Y    REP        #$30                 ; '%2Y' : Copy 2 bytes from address [Y]
ALL_PrepareNum2Y_1  LDA        $CCCC
                    XBA                             ; -> Big Endian
                    STA        ALL_ParamValueHex 
                    JSR        ALL_InsertHexaData2  ; Add 2 bytes in the String                    
                    JMP        ALL_Prepare_0
*--
                    mx         %10
ALL_PrepareNumUnk_2 STA        ALL_LogBuffer+2,Y    ; Unknown Sequence %2 sequence
                    LDA        #$25                 ; '%'
                    STA        ALL_LogBuffer,Y
                    LDA        #$32                 ; '2'
                    STA        ALL_LogBuffer+1,Y
                    INY
                    INY
                    INY
                    JMP        ALL_Prepare_1
*----
ALL_PrepareNum_4    LDA        $AAAA,X              ; %4 : Insert 32 bit from Address X / Y
                    INX
                    CMP        #$59                 ; 'Y'
                    BEQ        ALL_PrepareNum4Y
                    CMP        #$58                 ; 'X'
                    BNE        ALL_PrepareNumUnk_4
*--     
ALL_PrepareNum4X    REP        #$30                 ; '%4X' : Copy 4 bytes from address [X]
ALL_PrepareNum4X_1  LDA        $BBBB            
                    XBA                             ; -> Big Endian    
                    STA        ALL_ParamValueHex+2 
ALL_PrepareNum4X_2  LDA        $BBBB                ; [X+2]
                    XBA                             ; -> Big Endian    
                    STA        ALL_ParamValueHex                     
                    JSR        ALL_InsertHexaData4  ; Add 4 bytes in the String                    
                    JMP        ALL_Prepare_0
*--     
ALL_PrepareNum4Y    REP        #$30                 ; '%4Y' : Copy 4 bytes from address [Y]
ALL_PrepareNum4Y_1  LDA        $CCCC
                    XBA                             ; -> Big Endian
                    STA        ALL_ParamValueHex+2
ALL_PrepareNum4Y_2  LDA        $CCCC                ; [Y+2]
                    XBA                             ; -> Big Endian
                    STA        ALL_ParamValueHex                     
                    JSR        ALL_InsertHexaData4  ; Add 4 bytes in the String                    
                    JMP        ALL_Prepare_0
*--
                    mx         %10
ALL_PrepareNumUnk_4 STA        ALL_LogBuffer+2,Y    ; Unknown Sequence %4 sequence
                    LDA        #$25                 ; '%'
                    STA        ALL_LogBuffer,Y
                    LDA        #$34                 ; '4'
                    STA        ALL_LogBuffer+1,Y
                    INY
                    INY
                    INY
                    JMP        ALL_Prepare_1
*----                 
ALL_PrepareString   LDA        $AAAA,X              ; %S : Insert String from Address X / Y
                    INX
                    CMP        #$59                 ; 'Y'
                    BEQ        ALL_PrepareStrY
                    CMP        #$58                 ; 'X'
                    BNE        ALL_PrepareStrUnk
*--     
ALL_PrepareStrX     PHX                             ; '%sX' : Copy String from address [X]
ALL_PrepareStrX_1   LDX        #$BBBB
                    JSR        ALL_InsertString     ; Add StringLength byte in the String
                    PLX
                    JMP        ALL_Prepare_1
*--     
ALL_PrepareStrY     PHX                             ; '%sY' : Copy String from address [Y]
ALL_PrepareStrY_1   LDX        #$CCCC
                    JSR        ALL_InsertString     ; Add StringLength bytes in the String
                    PLX
                    JMP        ALL_Prepare_1
*--
ALL_PrepareStrUnk   STA        ALL_LogBuffer+2,Y    ; Unknown Sequence %S sequence
                    LDA        #$25                 ; '%'
                    STA        ALL_LogBuffer,Y
                    LDA        #$53                 ; 'S'
                    STA        ALL_LogBuffer+1,Y
                    INY
                    INY
                    INY
                    JMP        ALL_Prepare_1                    
*----
ALL_PrepareEnd      LDA        #$0D                 ; Add the \r at the end of the Log Line
                    STA        ALL_LogBuffer,Y
                    INY
                    REP        #$30
                    TYA                             ; Compute Log Line Length
                    CLC
                    ADC        #$0014               ; Time/Date length = 20 bytes
                    STA        ALL_LogLength

*----  Write Line Log to LogFile.txt file  ----
ALL_Open            LDA        ALL_LogFilePath      ; File Path
                    STA        ALL_gsosOPEN+4
                    LDA        #$0002               ; Open access Mode = Write
                    STA        ALL_gsosOPEN+8     
                    JSL        GSOS                 ; Open File
                    dw         $2010                ; Code for command 'Open'
                    adrl       ALL_gsosOPEN         ; Parameters Table Address (Open in Read Mode)
                    BCS        ALL_ErrorEnd
                    LDA        ALL_gsosOPEN+2       ; File Descriptor
                    STA        ALL_gsosSETMARK+2
                    STA        ALL_gsosWRITE+2
                    STA        ALL_gsosCLOSE+2
*--
ALL_SetMark         LDA        #$0001               ; EOF
                    STA        ALL_gsosSETMARK+4
                    JSL        GSOS                 ; Set Mark to End
                    dw         $2016                ; Code for command 'SetMark'
                    adrl       ALL_gsosSETMARK      ; Parameters Table Address   
                    BCS        ALL_Error
*--
ALL_Write           LDA        #ALL_TimeBuffer      ; Time Data Buffer
                    STA        ALL_gsosWRITE+4
                    LDA        ALL_LogLength        ; Line Length = Time/Date + Log Line
                    STA        ALL_gsosWRITE+8
                    STZ        ALL_gsosWRITE+10
                    JSL        GSOS                 ; Write Time / Date
                    dw         $2013                ; Code for command 'Write'
                    adrl       ALL_gsosWRITE        ; Parameters Table Address   
                    BCS        ALL_Error
*--                 
ALL_Close           JSL        GSOS                 ; Close File
                    dw         $2014                ; Code for command 'Close'
                    adrl       ALL_gsosCLOSE        ; Parameters Table Address 
*--
ALL_End             RTS
*--
ALL_Error           JSL        GSOS                 ; Close File
                    dw         $2014                ; Code for command 'Close'
                    adrl       ALL_gsosCLOSE        ; Parameters Table Address 
                    SEC                             ; Set Error Flag
*--
ALL_ErrorEnd        RTS
               
*----------   Log File Variables   ---------     

ALL_LogFilePath     HEX        0000                 ; Log File Path Address
ALL_ParamValueHex   HEX        00000000             ; Hexadecimal Value copied from Address (Big Endian)
ALL_ParamValueAscii HEX        0000000000000000     ; Ascii Value (Big Endian)
ALL_LogLength       HEX        0000                 ; Log Buffer Length
ALL_TimeBuffer      ds         20                   ; 20 bytes for Ascii Time/Date
ALL_LogBuffer       ds         256                  ; Log Line, 256 char Max

TableHexAscii       ASC        '000102030405060708090A0B0C0D0E0F'
                    ASC        '101112131415161718191A1B1C1D1E1F'
                    ASC        '202122232425262728292A2B2C2D2E2F'
                    ASC        '303132333435363738393A3B3C3D3E3F'
                    ASC        '404142434445464748494A4B4C4D4E4F'
                    ASC        '505152535455565758595A5B5C5D5E5F'
                    ASC        '606162636465666768696A6B6C6D6E6F'
                    ASC        '707172737475767778797A7B7C7D7E7F'
                    ASC        '808182838485868788898A8B8C8D8E8F'
                    ASC        '909192939495969798999A9B9C9D9E9F'
                    ASC        'A0A1A2A3A4A5A6A7A8A9AAABACADAEAF'
                    ASC        'B0B1B2B3B4B5B6B7B8B9BABBBCBDBEBF'
                    ASC        'C0C1C2C3C4C5C6C7C8C9CACBCCCDCECF'
                    ASC        'D0D1D2D3D4D5D6D7D8D9DADBDCDDDEDF'
                    ASC        'E0E1E2E3E4E5E6E7E8E9EAEBECEDEEEF'
                    ASC        'F0F1F2F3F4F5F6F7F8F9FAFBFCFDFEFF'

*----------   GS/OS Calls Variables   ---------
ALL_gsosOPEN        dw         3                    ; Number of parameters (Open in Write Mode)
                    ds         2                    ; +2 File Descriptor
                    adrl       LogFilePath          ; +4 File Pathname (init with an existing file path, so we only have to change 2 bytes)
                    ds         2                    ; +8 Request Access (1=Read,2=Write)

ALL_gsosSETMARK     dw         3                    ; Number of parameters
                    ds         2                    ; +2 File Descriptor
                    ds         2                    ; +4 Base
                    ds         4                    ; +6 Displacement

ALL_gsosWRITE       dw         5                    ; Number of parameters
                    ds         2                    ; +2  File Descriptor
                    adrl       LOG_LINE_1           ; +4  Data Buffer Address (init with an existing string path, so we only have to change 2 bytes)
                    ds         4                    ; +8  Data Size (Byte)
                    ds         4                    ; +12 Number of byte written to disk
                    ds         2                    ; +16 Cache Priority (#$0000 = No Cache)

ALL_gsosCLOSE       dw         1                    ; Number of parameters
                    ds         2                    ; +2 File Descriptor
                    
*----------   Insert Hexa Data into String   ---------
                    mx         %00
ALL_InsertHexaData1 PHX                             ; Copy 1 Byte in Hexa Ascii
*--
                    LDA        ALL_ParamValueHex    ; Byte #0
                    AND        #$00FF
                    ASL
                    TAX
                    LDA        TableHexAscii,X
                    STA        ALL_LogBuffer,Y
                    INY
                    INY
*--
                    PLX
                    RTS
*----
ALL_InsertHexaData2 PHX                             ; Copy 2 Bytes in Hexa Ascii
*--
                    LDA        ALL_ParamValueHex    ; Byte #0
                    AND        #$00FF
                    ASL
                    TAX
                    LDA        TableHexAscii,X
                    STA        ALL_LogBuffer,Y
                    INY
                    INY
*--
                    LDA        ALL_ParamValueHex+1  ; Byte #1
                    AND        #$00FF
                    ASL
                    TAX
                    LDA        TableHexAscii,X
                    STA        ALL_LogBuffer,Y
                    INY
                    INY      
*--              
                    PLX
                    RTS
*----
ALL_InsertHexaData4 PHX                             ; Copy 4 Bytes in Hexa Ascii
*--
                    LDA        ALL_ParamValueHex    ; Byte #0
                    AND        #$00FF
                    ASL
                    TAX
                    LDA        TableHexAscii,X
                    STA        ALL_LogBuffer,Y
                    INY
                    INY
*--
                    LDA        ALL_ParamValueHex+1  ; Byte #1
                    AND        #$00FF
                    ASL
                    TAX
                    LDA        TableHexAscii,X
                    STA        ALL_LogBuffer,Y
                    INY
                    INY 
*--     
                    LDA        ALL_ParamValueHex+2  ; Byte #2
                    AND        #$00FF
                    ASL
                    TAX
                    LDA        TableHexAscii,X
                    STA        ALL_LogBuffer,Y
                    INY
                    INY
*--  
                    LDA        ALL_ParamValueHex+3  ; Byte #3
                    AND        #$00FF
                    ASL
                    TAX
                    LDA        TableHexAscii,X
                    STA        ALL_LogBuffer,Y
                    INY
                    INY
*--          
                    PLX
                    RTS
                    
*----------   Insert SubString into String   ---------
                    mx          %10
ALL_InsertString    STX         ALL_InsertStr_1+1   ; Copy StringLength bytes
*-
                    LDX         #$0000
ALL_InsertStr_1     LDA         $DDDD,X
                    BEQ         ALL_InsertStringEnd
                    STA         ALL_LogBuffer,Y
                    INY
                    INX     
                    BRA         ALL_InsertStr_1
*-
ALL_InsertStringEnd RTS
                    
                    
********************************************************************************************************************
*  UnpackLZ4() :  Unpack LZ4 data buffer (IN:A=BankSrcDst,IN:X=HeaderSize,IN:Y=PackDataSize,OUT:A=UnpackDataSize). *
********************************************************************************************************************
UnpackLZ4           STA        LZ4_Literal_3+1      ; Uncompress a LZ4 Packed Data buffer (64 KB max)
                    STX        LZ4_Init_1+1         ; X = LZ4 Data
                    STX        LZ4_Init_2+1         
                    SEP        #$20                 ; A = Bank Src,Bank Dst
                    STA        LZ4_Match_5+1        
                    STA        LZ4_Match_5+2
                    XBA                             ;  => Return in A the length of unpacked Data
                    STA        LZ4_ReadToken+3  
                    STA        LZ4_Match_1+3    
                    STA        LZ4_GetLength_1+3
                    REP        #$30
                    TYA                             ; Y = Pack Data Size
                    CLC
LZ4_Init_1          ADC        #$AAAA               ; LZ4 Data offset
                    STA        LZ4_Limit+1          ; Last packed byte offset
LZ4_Init_2          LDA        #$AAAA               ; LZ4 Data
                    CLC
                    ADC        #$0010               ; LZ4 Header Size = 16 bytes
                    TAX                             ; X = First packed byte                 
*--
                    LDY        #BUFFER32KB          ; Init Target unpacked Data offset (#$0000 if dedicated uncompress buffer)
LZ4_ReadToken       LDAL       $AA0000,X            ; Read Token Byte
                    INX
                    STA        LZ4_Match_2+1
*-----------
LZ4_Literal         AND        #$00F0               ; >>> Process Literal Bytes <<<
                    BEQ        LZ4_Limit            ; No Literal
                    CMP        #$00F0
                    BNE        LZ4_Literal_1
                    JSR        LZ4_GetLengthLiter   ; Compute Literal Length with next bytes
                    BRA        LZ4_Literal_2
LZ4_Literal_1       LSR                             ; Literal Length use the 4 bit
                    LSR
                    LSR
                    LSR
*--
LZ4_Literal_2       DEC                             ; Copy A+1 Bytes
LZ4_Literal_3       MVN        $AA,$BB              ; Copy Literal Bytes from packed data buffer
                    PHK                             ; X and Y are auto incremented
                    PLB
*-----------
LZ4_Limit           CPX        #$AAAA               ; End Of Packed Data buffer ?
                    BEQ        LZ4_End
*-----------
LZ4_Match           TYA                             ; >>> Process Match Bytes <<<
                    SEC
LZ4_Match_1         SBCL       $AA0000,X            ; Match Offset
                    INX
                    INX
                    STA        LZ4_Match_4+1
*--
LZ4_Match_2         LDA        #$0000               ; Current Token Value
                    AND        #$000F
                    CMP        #$000F
                    BNE        LZ4_Match_3
                    JSR        LZ4_GetLengthMatch   ; Compute Match Length with next bytes
LZ4_Match_3         CLC
                    ADC        #$0003               ; Minimum Match Length is 4 (-1 for the MVN)
*--
                    PHX
LZ4_Match_4         LDX        #$AAAA               ; Match Byte Offset
LZ4_Match_5         MVN        $BB,$BB              ; Copy Match Bytes from unpacked data buffer
                    PHK                             ; X and Y are auto incremented
                    PLB
                    PLX
*-----------
                    BRA        LZ4_ReadToken
*-----------
LZ4_GetLengthLiter  LDA        #$000F               ; Compute Variable Length (Literal or Match)
LZ4_GetLengthMatch  STA        LZ4_GetLength_2+1
LZ4_GetLength_1     LDAL       $AA0000,X            ; Read Length Byte
                    INX
                    AND        #$00FF
                    CMP        #$00FF
                    BNE        LZ4_GetLength_3
                    CLC
LZ4_GetLength_2     ADC        #$000F
                    STA        LZ4_GetLength_2+1
                    BRA        LZ4_GetLength_1
LZ4_GetLength_3     ADC        LZ4_GetLength_2+1
                    RTS
*-----------
LZ4_End             TYA
                    SEC
                    SBC        #BUFFER32KB          ; A = Length of Unpack Data
                    RTS
                 
                 
**********************************************************
*  FadeOut() :  Fade Out Screen to black + Erase Screen. *
**********************************************************
FadeOut             LDY        #$000F               ; 16 Tones
*--
FOUT_1              LDX        #$01FE               ; 16 Palettes of 16 colors
*--
FOUT_2              LDAL       $019E00,X            ; Blue
                    AND        #$000F
                    BEQ        FOUT_3
                    LDAL       $019E00,X
                    SEC
                    SBC        #$0001
                    STAL       $019E00,X
FOUT_3              LDAL       $019E00,X            ; Green
                    AND        #$00F0
                    BEQ        FOUT_4
                    LDAL       $019E00,X
                    SEC
                    SBC        #$0010
                    STAL       $019E00,X
FOUT_4              LDAL       $019E00,X            ; Red
                    AND        #$0F00
                    BEQ        FOUT_5
                    LDAL       $019E00,X
                    SEC
                    SBC        #$0100
                    STAL       $019E00,X
*--
FOUT_5              DEX
                    DEX
                    BPL        FOUT_2
*--                 
FOUT_6              JSR        WaitForNextVBL       ; Tempo (1/60 sec)
*--
                    DEY
                    BPL        FOUT_1
*-----------
                    LDX        #$7FFE               ; Erase Screen
                    LDA        #$0000
FOUT_7              STAL       $012000,X
                    DEX
                    DEX
                    BPL        FOUT_7
*--         
                    RTS


*************************************************************************************
*  FadeIN() :  Fade In Screen from Black to Picture (IN:A=@Picture,IN:Y=FFFF/0000). *
*************************************************************************************
FadeIN              STA        FIN_1+1              ; Initialization
                    CLC
                    ADC        #$7E00               ; Palette
                    STA        FIN_5+1
                    STA        FIN_8+1
                    STA        FIN_11+1
                    CPY        #$FFFF               ; If Y=FFFF, no Picture Data Copy
                    BEQ        FIN_2
*--
                    LDX        #$7DFE               ; Copy Pixel + SCB 
FIN_1               LDAL       $062000,X
                    STAL       $012000,X
                    DEX
                    DEX
                    BPL        FIN_1
*--
FIN_2               LDY        #$000F               ; 16 Tones
*--
FIN_3               LDX        #$01FE               ; 16 Palettes of 16 colors
*--
FIN_4               LDAL       $019E00,X            ; Blue
                    AND        #$000F
                    STA        FIN_6+1
FIN_5               LDAL       $069E00,X
                    AND        #$000F
FIN_6               CMP        #$AAAA
                    BEQ        FIN_7                    
                    LDAL       $019E00,X
                    CLC
                    ADC        #$0001
                    STAL       $019E00,X
*--
FIN_7               LDAL       $019E00,X            ; Green
                    AND        #$00F0
                    STA        FIN_9+1
FIN_8               LDAL       $069E00,X
                    AND        #$00F0
FIN_9               CMP        #$AAAA
                    BEQ        FIN_10
                    LDAL       $019E00,X
                    CLC
                    ADC        #$0010
                    STAL       $019E00,X
*--
FIN_10              LDAL       $019E00,X            ; Red
                    AND        #$0F00
                    STA        FIN_12+1
FIN_11              LDAL       $069E00,X
                    AND        #$0F00
FIN_12              CMP        #$AAAA
                    BEQ        FIN_13
                    LDAL       $019E00,X
                    CLC
                    ADC        #$0100
                    STAL       $019E00,X
*--
FIN_13              DEX
                    DEX
                    BPL        FIN_4
*--
FIN_14              JSR        WaitForNextVBL       ; Tempo (1/60 sec)
*-- 
                    DEY
                    BPL        FIN_3
                    RTS


**********************************************************
*  WaitForNextVBL() :  Wait for one VBL to be completed. *
**********************************************************
WaitForNextVBL      LDA        #$0096               ; Wait for the Spot at Line #150
                    JSR        WaitForSpot
WFNVBL_1            LDAL       RDVBLBAR-1           ; Wait For VBL
                    BPL        WFNVBL_1
                    RTS
                    
                    
********************************************************************************************
*  WaitForSpot() :  Wait for the Video Spot to cross a screen line (IN:A=Line[0000-00C7]). *
********************************************************************************************
WaitForSpot         LSR                             ; SHR Line # / 2 : 00-63
                    CLC
                    ADC        #$0080               ; C02E Line Code : 80-E3
                    SEP        #$20
                    STA        WFS_2+1
*--
WFS_1               LDAL       VERTCNT              ; Current Code [7D-FF]
WFS_2               CMP        #$AA                 ; Expected Line Code
                    BNE        WFS_1
*--
                    REP        #$30
                    RTS
                  

****************************************************************
*  IncBorderColor() :  Increment Border Color (Debug Purpose). *
****************************************************************
IncBorderColor      SEP        #$20
                    LDAL       CLOCKCTL             ; Current Border Color
                    STA        IncBorderColor_1+1
                    INC
                    AND        #$0F                 ; New Color
                    STA        IncBorderColor_2+1
IncBorderColor_1    LDA        #$AA
                    AND        #$F0
IncBorderColor_2    ORA        #$AA
                    STAL       CLOCKCTL             ; New Border Color
                    REP	       #$30
                    RTS


*******************************************************************
*  SetBorderColor() :  Increment Border Color (IN:A=BorderColor). *
*******************************************************************
SetBorderColor      SEP        #$20
                    AND        #$0F
                    STA        SetBorderColor_1+1
                    LDAL       CLOCKCTL             ; Current Border Color
                    AND        #$F0
SetBorderColor_1    ORA        #$AA
                    STAL       CLOCKCTL             ; New Border Color
                    REP	       #$30
                    RTS


******************************************************************
*  Set40ColTextPage1() :  Display Text Page 1 in 40 column mode. *
******************************************************************
Set40ColTextPage1  	SEP        #$20
                    LDA        #$01
                    STAL       NEWVIDEO             ; Display Text Page
                    STAL       CLR80VID             ; Enforce 40 column mode
                    REP	       #$30
                    RTS


*********************************************************
*  ClearTextPage1() :  Clear the Screen of Text Page 1. *
*********************************************************
ClearTextPage1      LDY        #$002E               ; 24 lines
CTP1_1              LDA        TEXTPAGELINEADDR,Y   ; Address of the Line
                    STA        CTP1_2+1
*--
                    LDX        #$0026               ; 40 characters wide
                    LDA        #$A0A0               ; Space x 2
CTP1_2              STAL       $E0AAAA,X            ; Page Text 1 is located in $E0/0400
                    DEX
                    DEX
                    BPL        CTP1_2
*--                    
                    DEY
                    DEY
                    BPL        CTP1_1
                    RTS


***************************************************************************
*  BuildTextScreen() :  Display Text information on Screen (IN:A=@Table). *
***************************************************************************
BuildTextScreen     STA        BTS_1+1              ; Table of Line Information Address
                    LDY        #$0000
*--
BTS_1               LDA        $AAAA,Y              ; @ Line Information
                    BEQ        BTS_End
                    STA        BTS_2+1              ; @ Y
                    INC
                    INC
                    STA        BTS_3+1              ; @ X
                    INC
                    INC
                    STA        BTS_4+1              ; @ String
*--                    
BTS_2               LDA        $AAAA                ; @ Y
                    ASL
                    TAX
                    LDA        TEXTPAGELINEADDR,X 
                    CLC
BTS_3               ADC        $AAAA                ; @ X
                    STA        BTS_5+1              ; Address in Text Page where to display the String
*--
                    SEP        #$30                 ; Display Line High-Ascii value
                    LDX        #$00
BTS_4               LDA        $AAAA,X              ; @ String
                    BEQ        BTS_6
BTS_5               STAL       $E0AAAA,X            ; Page Text 1 is located in $E0/0400
                    INX
                    BRA        BTS_4
BTS_6               REP        #$30
*--
                    INY                             ; Next Line
                    INY
                    BRA        BTS_1
*--                    
BTS_End             RTS

*-------------------------------- Misc Tables ----------------------------------
                    
TEXTPAGELINEADDR    DA         $0400,$0480,$0500,$0580,$0600,$0680,$0700,$0780
                    DA         $0428,$04A8,$0528,$05A8,$0628,$06A8,$0728,$07A8      
                    DA         $0450,$04D0,$0550,$05D0,$0650,$06D0,$0750,$07D0


***********************************************************************************
*  DisplayString() :  Display a String on the Screen (IN:X=@String,IN:Y=@Screen). *
***********************************************************************************
DisplayString       STX        DS_String+1          ; String Address
                    STY        DS_Screen+1          ; Screen Address
                    STZ        DS_Index+1
*--
DS_Index            LDX        #$0000                     
DS_String           LDA        $AAAA,X              ; String Data
                    AND        #$00FF
                    STA        DS_Char_2+1
                    BEQ        DS_End               ; End of String
                    INC        DS_Index+1
*--
                    LDX        #$0000               ; Search character into 
DS_Char_1           LDAL       Alpha_Char,X         ; authorized characters list
                    AND        #$00FF
                    BEQ        DS_Index             ; Unknown character
DS_Char_2           CMP        #$AAAA
                    BEQ        DS_Display
                    INX
                    BRA        DS_Char_1
*--                    
DS_Display          TXA                             ; Character Bitmap Address
                    ASL
                    TAX
                    LDAL       Alpha_Tab,X
                    TAX                             ; Bitmap Address
DS_Screen           LDY        #$AAAA               ; Screen Address
                    LDA        #$030A               ; Width4 / Heigh
                    SEC                             ; E1/2000
                    JSL        CopyTransBitmap
*--
                    LDA        DS_Screen+1          ; New Screen Address
                    CLC
                    ADC        #$0004
                    STA        DS_Screen+1
                    BRA        DS_Index
*--   
DS_End              RTS


****************************************************************
*  PauseGame() :  Pause Audio + Video waiting for a Key press. *
****************************************************************
PauseGame           JSR        EnableDisableAudio  ; Pause Audio
*--
                    LDX        #$1FE               ; Backup Palette
PG_BP               LDAL       $019E00,X
                    STA        BUFFERPALETTE,X
                    DEX
                    DEX
                    BPL        PG_BP
*--
                    LDA        #$0000              ; For TAY 8 bits -> 16 bits
                    SEP        #$20
                    LDX        #$1FF               ; Dark Palette
PG_DP               LDAL       $019E00,X
                    TAY
                    LDA        PG_DarkPalette,Y    ; Divide by 2 RGB components
                    STAL       $019E00,X
                    DEX
                    BPL        PG_DP                    
                    REP        #$30
*--
                    JSR        WaitForKey          ; Wait for Key press
*--
                    LDX        #$1FE               ; Restaure Palette
PG_RP               LDA        BUFFERPALETTE,X
                    STAL       $019E00,X
                    DEX
                    DEX
                    BPL        PG_RP
*--
                    JSR        EnableDisableAudio  ; Restart Audio
*--
                    RTS
                    
*----------   PauseGame Variables   ---------

PG_DarkPalette      HEX        00,00,01,01,02,02,03,03,04,04,05,05,06,06,07,07
                    HEX        00,00,01,01,02,02,03,03,04,04,05,05,06,06,07,07
                    HEX        10,10,11,11,12,12,13,13,14,14,15,15,16,16,17,17
                    HEX        10,10,11,11,12,12,13,13,14,14,15,15,16,16,17,17
                    HEX        20,20,21,21,22,22,23,23,24,24,25,25,26,26,27,27
                    HEX        20,20,21,21,22,22,23,23,24,24,25,25,26,26,27,27
                    HEX        30,30,31,31,32,32,33,33,34,34,35,35,36,36,37,37
                    HEX        30,30,31,31,32,32,33,33,34,34,35,35,36,36,37,37
                    HEX        40,40,41,41,42,42,43,43,44,44,45,45,46,46,47,47
                    HEX        40,40,41,41,42,42,43,43,44,44,45,45,46,46,47,47
                    HEX        50,50,51,51,52,52,53,53,54,54,55,55,56,56,57,57
                    HEX        50,50,51,51,52,52,53,53,54,54,55,55,56,56,57,57
                    HEX        60,60,61,61,62,62,63,63,64,64,65,65,66,66,67,67
                    HEX        60,60,61,61,62,62,63,63,64,64,65,65,66,66,67,67
                    HEX        70,70,71,71,72,72,73,73,74,74,75,75,76,76,77,77
                    HEX        70,70,71,71,72,72,73,73,74,74,75,75,76,76,77,77


**********************************************************************
*  EnableDisableAudio() :  Enable / Disable Music and Sound effects. *
**********************************************************************
EnableDisableAudio  LDA        SoundToolError       ; No Sound ?
                    BEQ        EDA_Init
                    JMP        EDA_End
*--  
EDA_Init            SEP        #$30
                    SEI                             ; Stop Interruption
*--
                    LDA        EDA_STATUS
                    BEQ        EDA_DisableAudio
*--------
EDA_EnableAudio     STZ        EDA_STATUS           ; Enable Audio
*--
EDA_EA_Ready        LDAL       SOUNDCTL
                    BMI        EDA_EA_Ready
*--
                    LDAL       CTRLPNLSNDVOL        ; Control Panel Sound volume
                    AND        #$0F
                    STAL       SOUNDCTL
*--
                    LDX        #$00
EDA_Restaure        TXA
                    CLC
                    ADC        #$A0
                    STAL       SOUNDADRL
                    LDAL       EDA_SoundRegister,X  ; Restaure Sound Registers
                    STAL       SOUNDDATA
                    INX
                    CPX        #$20
                    BNE        EDA_Restaure
*--
                    BRA        EDA_End
*--------
EDA_DisableAudio    INC        EDA_STATUS           ; Disable Audio
*--
EDA_DA_Ready        LDAL       SOUNDCTL
                    BMI        EDA_DA_Ready
                    LDAL       CTRLPNLSNDVOL        ; Control Panel Sound volume
                    AND        #$0F
                    STAL       SOUNDCTL
*----
                    LDX        #$00
EDA_DA_Backup       TXA
                    CLC
                    ADC        #$A0
                    STAL       SOUNDADRL
                    LDAL       SOUNDDATA
                    LDAL       SOUNDDATA
                    STAL       EDA_SoundRegister,X  ; Backup Sound Registers
                    TXA
                    CLC
                    ADC        #$A1
                    STAL       SOUNDADRL
                    LDAL       SOUNDDATA
                    LDAL       SOUNDDATA
                    STAL       EDA_SoundRegister+1,X
*--
                    LDY        #$02                 ; x2 for the Loop Songs
EDA_DA_Stop         TXA
                    CLC
                    ADC        #$A0
                    STAL       SOUNDADRL
                    LDA        #%0000_0001          ; Stop Oscillator
                    STAL       SOUNDDATA
                    TXA
                    CLC
                    ADC        #$A1
                    STAL       SOUNDADRL
                    LDAL       #%0000_0001
                    STAL       SOUNDDATA
                    DEY
                    BNE        EDA_DA_Stop
*--
                    INX
                    INX
                    CPX        #$20
                    BNE        EDA_DA_Backup
*----
EDA_End             CLI                             ; Enable Interruption
                    REP        #$30
                    RTS

*----------   EnableDisableAudio Variables   ---------
EDA_STATUS          HEX        00                   ; Enable (00) / Disable (01)
EDA_SoundRegister   ds         32                   ; Sound Register values
         
         
******************************************************         
*  MuteUnmuteAudio() :  Mute / Unmute Sound effects. *          
******************************************************
MuteUnmuteAudio     SEP        #$30                 ; Inverse Sound Status
*--
                    LDAL       CTRLPNLSNDVOL        ; Control Panel Sound Volume       
                    AND        #$0F                 ; Access Sound Register + Auto Increment Disabled           
                    STAL       SOUNDCTL
*--
                    LDA        MUA_STATUS
                    BEQ        MUA_Unmute
*--
MUA_Mute            INC        MUA_STATUS           ; Set Sound Level to Min : FF -> 00
                    BRA        MUA_Begin
MUA_Unmute          DEC        MUA_STATUS           ; Set Sound Level to Max : 00 -> FF
*--
MUA_Begin           LDX        #$40      
MUA_Loop            TXA
                    STAL       SOUNDADRL
                    LDA        MUA_STATUS           ; 00 or FF
                    STAL       SOUNDDATA
                    INX
                    CPX        #$60
                    BNE        MUA_Loop
*--
                    REP        #$30            
                    RTS  


***************************************************************         
*  GetMuteStatus() :  Get current Mute status (OUT:A=Status). *          
***************************************************************
GetMuteStatus       LDA        MUA_STATUS           ; Work in 8 and 16 bit
                    RTS

*----------   MuteUnmuteAudio / GetMuteStatus Variables   ---------
MUA_STATUS          HEX        FF,00                ; Unmute (FF) / Mute (00)
         

*************************************************
*  StopSound() :  Stop the first 4 oscillators. *
*************************************************
StopSound           LDA        SoundToolError       ; No Sound ?
                    BEQ        StopSound_Init
                    JMP        StopSound_End
*--
StopSound_Init      SEP        #$20
                    LDAL       CTRLPNLSNDVOL        ; Control Panel Sound Volume       
                    AND        #$0F                 ; Access Sound Register + Auto Increment Disabled           
                    STAL       SOUNDCTL
*--
                    LDY        #$0002
StopSound_1         LDA        #$A0                 ; Control Register for Oscillator #00
                    STAL       SOUNDADRL
                    LDA        #$03                 ; Canal #0=0000 / No Interrupt=0 / One-Shot=01 / Stop Sound=1
                    STAL       SOUNDDATA
*--
                    LDA        #$A1                 ; Control Register for Oscillator #01                 
                    STAL       SOUNDADRL
                    LDA        #$03                 ; Canal #0=0000 / No Interrupt=0 / One-Shot=01 / Stop Sound=1
                    STAL       SOUNDDATA
*--
                    LDA        #$A2                 ; Control Register for Oscillator #02                 
                    STAL       SOUNDADRL
                    LDA        #$03                 ; Canal #0=0000 / No Interrupt=0 / One-Shot=01 / Stop Sound=1
                    STAL       SOUNDDATA
*--
                    LDA        #$A3                 ; Control Register for Oscillator #03                 
                    STAL       SOUNDADRL
                    LDA        #$03                 ; Canal #0=0000 / No Interrupt=0 / One-Shot=01 / Stop Sound=1
                    STAL       SOUNDDATA
                    DEY
                    BPL        StopSound_1
*--
                    REP        #$30
StopSound_End       RTS


******************************************************************************************
*  HexDec2Hex() :  Convert a Decimal number into Hexadecimal (IN:A=NumDec,OUT:A=NumHex). *
******************************************************************************************
HexDec2Hex          STA        HD2H_H+1             ; Convert 032 as 0*100 + 3*10 + 2 = 000 + 01E + 002 = 020
                    STA        HD2H_D+1
                    AND        #$000F
                    STA        HD2H_U+1
*--
HD2H_H              LDA        #$AAAA
                    AND        #$0F00
                    XBA
                    ASL
                    TAX
                    LDA        HD2H_Tab100,X 
                    STA        HD2H_Hundred+1
*--
HD2H_D              LDA        #$AAAA
                    AND        #$00F0
                    LSR
                    LSR
                    LSR
                    LSR
                    TAX
                    LDA        HD2H_Tab10,X         ; Dozen
                    AND        #$00FF
*--
                    CLC
HD2H_U              ADC        #$AAAA               ; Unit
HD2H_Hundred        ADC        #$AAAA               ; Hundreds
*--
                    RTS                    

*----------   HexDec2Hex Variables   ---------
HD2H_Tab10          HEX        00,0A,14,1E,28,32,3C,46,50,5A,64,6E,78,82,8C,96          
HD2H_Tab100         HEX        0000,6400,C800,2C01,9001,F401,5802,BC02,2003,8403      


******************************************************************************************
*  Hex2HexDec() :  Convert a Hexadecimal number into Decimal (IN:A=NumHex,OUT:A=NumDec). *
******************************************************************************************
Hex2HexDec          AND        #$00FF               ; Convert $2A as 2*16+10 = 42 = $42
                    TAX
                    LDA        H2HD_Tab,X
                    AND        #$00FF
                    RTS                    

*----------   HexDec2Hex Variables   ---------
H2HD_Tab            HEX        00,01,02,03,04,05,06,07,08,09,10,11,12,13,14,15
                    HEX        16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31
                    HEX        32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47
                    HEX        48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63
                    HEX        64,65,66,67,68,69,70,71,72,73,74,75,76,77,78,79
                    HEX        80,81,82,83,84,85,86,87,88,89,90,91,92,93,94,95
                    HEX        96,97,98,99          


*******************************************************************
*  HBEntryPoint() :  HeartBeat Interruption Entry Point function. *
*******************************************************************
HBEntryPoint        ds         4
                    dw         1                    ; 1/60 seconds
                    dw         $A55A                ; Signature
*--
                    CLC
                    XCE
                    REP        #$30
*--
                    LDA        #$0001               ; Reset Timer
                    STA        HBEntryPoint+4
*--
                    LDAL       HBCounter            ; Increment Counter
                    INC
                    STAL       HBCounter
*--
                    SEP        #$30
                    CLC
                    RTL
                    mx         %00
                    
                     
*****************************************************************
*  HBResetCounter() :  Reset Counter of HeartBeat Interruption. *
*****************************************************************
HBResetCounter      STZ        HBCounter            ; Set VBL counter to 0
                    STZ        HBCounter
                    STZ        HBCounter 
                    STZ        HBCounter 
                    RTS


********************************************************************************
*  HBGetCounter() :  Get HeartBeat Interruption Counter value (OUT:A=Counter). *
********************************************************************************
HBGetCounter        LDA        HBCounter            ; Get VBL counter
                    RTS

*----------   HBEntryPoint / HBResetCounter / HBGetCounter Variables   ---------
HBCounter           HEX        0000


**********************************************************************
*  SwitchTwilightOff/On() :  Turn Off / On Twilight II screen saver. *
**********************************************************************
SwitchTwilightOff   LDAL       $E11600
                    STA        $00
                    LDAL       $E11602
                    STA        $02
*--
STOff_1             LDY        #$0008
                    LDA        [$00],y
                    LDX        #$117A               ; offV1
                    CMP        #$49BF               ; lenV1
                    BEQ        STOff_2
                    LDX        #$154C               ; offV2
                    CMP        #$539A               ; lenV2
                    BNE        STOff_3
*----
STOff_2             STX        TWILIGHT_off
                    LDA        [$00]
                    STA        $04
                    STA        TWILIGHT_ptr
                    LDY        #$0002
                    LDA        [$00],y
                    STA        $06
                    STA        TWILIGHT_ptr+2
                    TXY
                    LDA        [$04],y
                    CMP        #$0EF0
                    BNE        STOff_3
                    LDA        #$0E80
                    STA        [$04],y
                    INC        TWILIGHT_fg
*--
                    RTS
*----
STOff_3             LDY        #$0010
                    LDA        [$00],y
                    TAX
                    INY
                    INY
                    LDA        [$00],y
                    STA        $02
                    TXA
                    STA        $00
                    LDA        $00
                    ORA        $02
                    BNE        STOff_1
*--
                    RTS
*--------------------------------------------------------
SwitchTwilightOn    LDA        TWILIGHT_fg
                    BNE        STon_1
                    RTS
*--
STon_1              LDA        TWILIGHT_ptr
                    STA        $04
                    LDA        TWILIGHT_ptr+2
                    STA        $06
                    LDY        TWILIGHT_off
                    LDA        #$0EF0
                    STA        [$04],y
*--
                    RTS

*----------   TWILIGHToff / TWILIGHTon Variables   ---------
TWILIGHT_ptr        DS         4
TWILIGHT_fg         DS         2
TWILIGHT_off        DS         2
                  
********************************************************************************